﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using ThinkGeo.MapSuite.Core;
using System.IO;

namespace _20160311_ThinkgeoDll8.Classes
{
    public class CoucheFactory
    {
        private List<Layer> layers;
        private double zoomMin;
        private double ZoomMax;

        public List<Layer> Layers
        {
            get { return layers; }
            private set { layers = value; }
        }

        public RectangleShape BoundingBox { get; private set; }

        public CoucheFactory(XmlCoucheCollection couches,List<ConstantsMap.TypeCouche> select)
        {
            layers = new List<Layer>();

            foreach (var c in couches.XmlCouches)
            {
                if (select.Contains(c.Type))
                {
                    switch (c.Type)
                    {
                        case ConstantsMap.TypeCouche.shp:
                            layers.Add(initialiserCoucheShp(c));                 
                        break;

                        case ConstantsMap.TypeCouche.raster:                
                            layers.Add(initialiserCoucheRaster(c));
                   
                        break;

                        case ConstantsMap.TypeCouche.sql:
                            layers.Add(initialiserCoucheSql(c));
                   
                        break;
                    }
                }
            }
        }

        private ShapeFileFeatureLayer initialiserCoucheShp(XmlCouche coucheXml)
        {
            var rep = OutilsOvLyr.Path("SHP\\") + coucheXml.Repertoire;
            ShapeFileFeatureLayer sh = new ShapeFileFeatureLayer(rep);
            sh.Name = coucheXml.Nom;
            ShapeFileFeatureSource.BuildIndexFile(rep, BuildIndexMode.DoNotRebuild);
            sh.RequireIndex = true;

            sh.ZoomLevelSet.ZoomLevel01.CustomStyles.Add(AreaStyles.CreateSimpleAreaStyle(GeoColor.SimpleColors.Transparent, ConstantsMap.RenvoyerCouleur(coucheXml.Couleur), 3));
            sh.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;
            sh.ZoomLevelSet.ZoomLevel01.CustomStyles.Add(TextStyles.CreateSimpleTextStyle(coucheXml.Libelle, "Arial", 10, DrawingFontStyles.Bold, ConstantsMap.RenvoyerCouleur(coucheXml.Couleur)));
            sh.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;

            if (coucheXml.CoucheBase)
            {
                zoomMin = sh.ZoomLevelSet.ZoomLevel01.Scale;
                ZoomMax = sh.ZoomLevelSet.ZoomLevel20.Scale;

                try
                {
                    sh.Open();
                    BoundingBox = sh.GetBoundingBox();
                }
                catch (Exception ex)
                {

                }
            }

            return sh;
        }

        private MultiGeoRasterLayer initialiserCoucheRaster(XmlCouche coucheXml)
        {
            var repertoireTxt = Path.Combine(OutilsOvLyr.PathRefTxtRaster, ConstantsMap.REFERENCETXT);
            MultiGeoRasterLayer multiRaster = new MultiGeoRasterLayer(repertoireTxt, 590591790.0, 1126.4644432067871, coucheXml.Nom);
            multiRaster.BuildReferenceFile(repertoireTxt, coucheXml.Repertoire, false);
            return multiRaster;
        }

        private MsSql2008FeatureLayer initialiserCoucheSql(XmlCouche coucheXml)
        {
            var layer = new MsSql2008FeatureLayer(coucheXml.Repertoire,coucheXml.SchemaTable.Split('.').Last(),coucheXml.FeatureIdColumn);
            layer.CustomGeometryColumnName = coucheXml.CustomGeometryColumnName;
            layer.SchemaName = coucheXml.SchemaTable.Split('.').First();
            layer.Srid = 2154;
            layer.Name = coucheXml.Libelle;
            layer.ZoomLevelSet.ZoomLevel01.CustomStyles.Add(AreaStyles.CreateSimpleAreaStyle(GeoColor.SimpleColors.Yellow, ConstantsMap.RenvoyerCouleur(coucheXml.Couleur), 3));
            layer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;
            layer.ZoomLevelSet.ZoomLevel01.CustomStyles.Add(TextStyles.CreateSimpleTextStyle(coucheXml.Libelle, "Arial", 10, DrawingFontStyles.Bold, GeoColor.SimpleColors.Black));
            layer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;

            return layer;
        }
    }
}
