﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using ThinkGeo.MapSuite.Core;
using ThinkGeo.MapSuite.WpfDesktopEdition;
using System.IO;
using _20160311_ThinkgeoDll8.Classes;
using _20160311_ThinkgeoDll8.Enums;


namespace _20160311_ThinkgeoDll8
{
    /// <summary>
    /// Logique d'interaction pour MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        private CoucheFactory factoryLayer;
        private CoucheFactory factoryRaster;

        public MainWindow()
        {
            InitializeComponent();
            var couchesACharger = chargerCouchesXml();
            factoryLayer = new CoucheFactory(couchesACharger, new List<ConstantsMap.TypeCouche>() { ConstantsMap.TypeCouche.shp, ConstantsMap.TypeCouche.sql });
            factoryRaster = new CoucheFactory(couchesACharger, new List<ConstantsMap.TypeCouche>() { ConstantsMap.TypeCouche.raster });
        }

        private void Map1_Loaded(object sender, RoutedEventArgs e)
        {

            Map1.MapUnit = GeographyUnit.Meter;                      
            //Map1.Overlays.Add(ConstantsMap.NOMOVERLAYRASTER + "_KEY", initialiserOverlayRaster(factoryRaster.Layers));
           // Map1.Overlays.Add(ConstantsMap.NOMOVERLAYLAYER + "_KEY", initialiserOverlayLayer(factoryLayer.Layers));

            var bbox = new RectangleShape(614999.75, 6670001.25, 619999.75, 6665001.25);
            Map1.CurrentExtent = bbox.GetBoundingBox();
            //Map1.CurrentExtent = factoryLayer.BoundingBox;
            //Map1.Refresh();
        }   

        private LayerOverlay initialiserOverlayRaster(IEnumerable<Layer> layers)
        {
            LayerOverlay ovLy = new LayerOverlay();            
            ovLy.TileHeight = 256; ovLy.TileWidth = 256; ovLy.TileBuffer = 2; ovLy.TileType = TileType.HybridTile;
            ovLy.TileCache = new FileBitmapTileCache(@"CacheRaster\", "ManualCache");
            ovLy.DrawingQuality = DrawingQuality.HighSpeed;
            ovLy.TileBuffer = 2;

            ovLy.Name = ConstantsMap.NOMOVERLAYRASTER;
            return initialiserOverlay(ovLy,layers);           
        }

        private LayerOverlay initialiserOverlayLayer(IEnumerable<Layer> layers)
        {
            LayerOverlay ovLy = new LayerOverlay();
            ovLy.TileType = TileType.SingleTile;         
            ovLy.Name = ConstantsMap.NOMOVERLAYLAYER;
            return initialiserOverlay(ovLy, layers);
        }

        private XmlCoucheCollection chargerCouchesXml()
        {
            var fichierXml = System.IO.Path.Combine(Directory.GetCurrentDirectory(),ConstantsMap.FICHIERXML);
            if (!File.Exists(fichierXml))
                throw new FileNotFoundException("Fichier Xml pour chargement des couches non trouvé");

            return XmlCoucheSerializer.DeserialiserXml(fichierXml);
        }

        private LayerOverlay initialiserOverlay(LayerOverlay ovLayer, IEnumerable<Layer> layers)
        {            
            foreach (var c in layers)
            {
                ovLayer.Layers.Add(c);
            }

            return ovLayer;
        }

        private void extendBtn_Click(object sender, RoutedEventArgs e)
        {
            Map1.CurrentExtent = factoryLayer.BoundingBox;
            Map1.Refresh();
        }

        private void closeBtn_Click(object sender, RoutedEventArgs e)
        {
            Map1.Overlays[0].Dispose();
            Map1.Overlays.Clear();
           // Map1.CurrentExtent = factoryLayer.BoundingBox;
            Map1.Refresh();
        }

        private void addBtn_Click(object sender, RoutedEventArgs e)
        {
            Map1.Overlays.Add(ConstantsMap.NOMOVERLAYRASTER + "_KEY", initialiserOverlayRaster(factoryRaster.Layers));
            Map1.Refresh();
         
        }

        private void runGCBtn_Click(object sender, RoutedEventArgs e)
        {
            GC.Collect();
            GC.WaitForPendingFinalizers();
        }

        private void zoomDpt17_Click(object sender, RoutedEventArgs e)
        {
            var lyr = factoryLayer.Layers.FirstOrDefault(p => p.Name.Contains("IlotCharenteM"));
            if (lyr != null)
            {
                Map1.CurrentExtent = lyr.GetBoundingBox();
                Map1.Refresh();
            }
            else
                MessageBox.Show("kjdfjls");
        }

        private void Raster_Click(object sender, RoutedEventArgs e)
        {
            var bbox = new RectangleShape(459999.75, 6560000.25, 464999.75, 6555000.25);
            Map1.CurrentExtent = bbox.GetBoundingBox();
            Map1.Refresh();
            GC.Collect();
        }

        private void Jp2_Click(object sender, RoutedEventArgs e)
        {
            //var bbox = new RectangleShape(362000, 6567000.25, 382000, 6547000.25);
            var bbox = new RectangleShape(614999.75, 6670001.25, 619999.75, 6665001.25);           
            Map1.CurrentExtent = bbox.GetBoundingBox();
            Map1.Refresh();
            GC.Collect();
        }

        private void layerSql_Click(object sender, RoutedEventArgs e)
        {
            var lyr = factoryLayer.Layers.FirstOrDefault(p => p.Name == "CartoDetourageId");
            if (lyr != null)
            {
                Map1.CurrentExtent = lyr.GetBoundingBox();
                Map1.Refresh();
            }
            else
                MessageBox.Show("kjdfjls");
        }

    }
}
